@@ -1,4 +1,4 @@
-from robot.output import LOGGER
+from robot.api import logger
 
 from base64 import b64encode
 from functools import wraps
@@ -51,8 +51,27 @@ class HTTP:
         # setup new http context
         self._post_process_request()
 
+    def _pre_process_request(self):
+
+        if len(self._request_headers.items()) > 0:
+            logger.debug("Request headers:")
+            for name, value in self._request_headers.items():
+                logger.debug("%s: %s" % (name, value))
+        else:
+            logger.debug("No request headers set")
+
+        if self._request_body is None:
+            logger.debug("No request body set")
+        else:
+            logger.debug("Request body:")
+            logger.debug(self._request_body)
+
     def _post_process_request(self):
 
+        if self._response != None:
+            self.log_response_headers('DEBUG')
+            self.log_response_body('DEBUG')
+
         # check flag set by "Next Request Should Succeed"
         if self._next_request_should == True:
             assert int(self.response.status[0:3]) < 400, \
@@ -107,6 +126,7 @@ class HTTP:
 
         `host` is the name of the host, optionally with port (e.g. 'google.com' or 'localhost:5984')
         """
+        logger.info("Host for next HTTP request set to '%s'" % host)
         self._app = livetest.TestApp(host)
 
     # request
@@ -120,6 +140,7 @@ class HTTP:
         """
         path = self._path_from_url_or_path(url)
 
+        self._pre_process_request()
         self._response = self._app.request(path, {}, self._request_headers,
                 method=verb.upper(),)
         self._post_process_request()
@@ -131,6 +152,7 @@ class HTTP:
         `url` is the URL relative to the server root, e.g. '/_utils/config.html'
         """
         path = self._path_from_url_or_path(url)
+        self._pre_process_request()
         self._response = self.app.head(path, {}, self._request_headers)
         self._post_process_request()
 
@@ -141,6 +163,7 @@ class HTTP:
         `url` is the URL relative to the server root, e.g. '/_utils/config.html'
         """
         path = self._path_from_url_or_path(url)
+        self._pre_process_request()
         self._response = self.app.get(path, {}, self._request_headers)
         self._post_process_request()
 
@@ -154,6 +177,7 @@ class HTTP:
         kwargs = {}
         if 'Content-Type' in self._request_headers:
             kwargs['content_type'] = self._request_headers['Content-Type']
+        self._pre_process_request()
         self._response = self.app.post(path, self._request_body or {}, self._request_headers, **kwargs)
         self._post_process_request()
 
@@ -167,6 +191,7 @@ class HTTP:
         kwargs = {}
         if 'Content-Type' in self._request_headers:
             kwargs['content_type'] = self._request_headers['Content-Type']
+        self._pre_process_request()
         self._response = self.app.put(path, self._request_body or {}, self._request_headers, **kwargs)
         self._post_process_request()
 
@@ -177,6 +202,7 @@ class HTTP:
         `url` is the URL relative to the server root, e.g. '/_utils/config.html'
         """
         path = self._path_from_url_or_path(url)
+        self._pre_process_request()
         self._response = self.app.delete(path, {}, self._request_headers)
         self._post_process_request()
 
@@ -276,7 +302,9 @@ class HTTP:
 
     def response_header_should_equal(self, header_name, expected):
         """
-        Fails if the value of response header `header_name` does not equal `expected`
+        Fails if the value of response header `header_name` does not equal
+        `expected`. Also fails if the last response does not have a
+        `header_name` header.
         """
         self.response_should_have_header(header_name)
         actual = self.response.headers[header_name]
@@ -286,7 +314,8 @@ class HTTP:
 
     def response_header_should_not_equal(self, header_name, not_expected):
         """
-        Fails if the value of response header 'header_name' equals 'expected'
+        Fails if the value of response header `header_name` equals `expected`
+        Also fails if the last response does not have a `header_name` header.
         """
         self.response_should_have_header(header_name)
         actual = self.response.headers[header_name]
@@ -300,8 +329,9 @@ class HTTP:
 
         Specify `log_level` (default: "INFO") to set the log level.
         """
+        logger.write("Response headers:", log_level)
         for name, value in self.response.headers.items():
-            LOGGER.write("%s: %s" % (name, value), log_level)
+            logger.write("%s: %s" % (name, value), log_level)
 
     # request headers
 
@@ -312,7 +342,7 @@ class HTTP:
         `header_name` is the name of the header, e.g. `User-Agent`
         `header_value` is the key of the header, e.g. `RobotFramework HttpLibrary (Mozilla/4.0)`
         """
-        LOGGER.write('Set request header "%s" to "%s"' % (header_name, header_value), 'INFO')
+        logger.info('Set request header "%s" to "%s"' % (header_name, header_value))
         self._request_headers[header_name] = header_value
 
     def set_basic_auth(self, username, password):
@@ -326,7 +356,7 @@ class HTTP:
         `password` is the password to use, e.g. 'open sesame'
         """
         credentials = "%s:%s" % (username, password)
-        LOGGER.write('Set basic auth to "%s"' % credentials, 'INFO')
+        logger.info('Set basic auth to "%s"' % credentials)
         self.set_request_header("Authorization", "Basic %s" % b64encode(credentials))
 
     # payload
@@ -340,7 +370,7 @@ class HTTP:
         | POST                       | /login                              |
         | Response Should Succeed  |                                     |
         """
-        LOGGER.write('Request body set to "%s".' % body, 'INFO')
+        logger.info('Request body set to "%s".' % body)
         self._request_body = body.encode("utf-8")
 
     def get_response_body(self):
@@ -372,7 +402,11 @@ class HTTP:
 
         Specify `log_level` (default: "INFO") to set the log level.
         """
-        LOGGER.write(self.response.body, log_level)
+        if self.response.body:
+            logger.write("Response body:", log_level)
+            logger.write(self.response.body, log_level)
+        else:
+            logger.debug("No response body received", log_level)
 
     # json
 
@@ -434,8 +468,11 @@ class HTTP:
 
         got = self.get_json_value(json_string, json_pointer)
 
-        assert got != expected_value, \
-               'JSON value "%s" does not equal "%s", but should have.' % (got, expected_value)
+        message = 'JSON value "%s" does not equal "%s"' % (got, expected_value)
+
+        assert got != expected_value, "%s, but should have." % message
+
+        logger.debug("%s." % message)
 
 
     @_with_json
@@ -458,7 +495,7 @@ class HTTP:
         Logs a pretty printed version of the JSON document `json_string`.
         """
         for line in json.dumps(json_string, indent=2).split('\n'):
-            LOGGER.write(line, log_level)
+            logger.write(line, log_level)
 
     # debug
 
